@@ -148,16 +148,19 @@ int refreshgroupcache() {
       continue;
     }
     std::string name(grp.gr_name);
+    users.clear();
     if (!GetUsersForGroup(name, &users, &error_code)) {
       syslog(LOG_ERR,
              "Error getting users for group %s (error_code %d), skipping.",
              grp.gr_name, error_code);
       continue;
     }
-    cache_file << grp.gr_name << ":" << grp.gr_passwd << ":" << grp.gr_gid << ":" << users.front();
-    users.erase(users.begin());
+    cache_file << grp.gr_name << ":" << grp.gr_passwd << ":" << grp.gr_gid << ":";
     for (int i = 0; i < (int)users.size(); i++) {
-      cache_file << "," << users[i];
+      if (i > 0) {
+        cache_file << ",";
+      }
+      cache_file << users[i];
     }
     cache_file << "\n";
   }
