@@ -406,23 +406,13 @@ TEST(ParseJsonToUsersTest, ParseJsonToUsersSucceeds) {
 
 // Test parsing a valid JSON response from the metadata server.
 TEST(ParseJsonToUsersTest, ParseJsonToUsersEmptyGroupSucceeds) {
-  string test_group_users = "{\"usernames\":[]}";
+  string test_group_users = "{\"nextPageToken\":\"0\"}";
 
   std::vector<string> users;
   ASSERT_TRUE(ParseJsonToUsers(test_group_users, &users));
   ASSERT_TRUE(users.empty());
 }
 
-// Test parsing malformed JSON responses.
-TEST(ParseJsonToUsersTest, ParseJsonToUsersFails) {
-  string test_group_users =
-      "{\"badstuff\":[\"user0001\",\"user0002\",\"user0003\",\"user0004\","
-      "\"user0005\"]}";
-
-  std::vector<string> users;
-  ASSERT_FALSE(ParseJsonToUsers(test_group_users, &users));
-}
-
 TEST(GetUsersForGroupTest, GetUsersForGroupSucceeds) {
   string response;
   long http_code;
@@ -437,7 +427,24 @@ TEST(GetUsersForGroupTest, GetUsersForGroupSucceeds) {
   ASSERT_EQ(errnop, 0);
 }
 
-TEST(FindGroupTest, FindGroupByGidSucceeds) {
+TEST(GetGroupByTest, GetGroupByNameSucceeds) {
+  string response;
+  long http_code;
+  ASSERT_TRUE(HttpGet("http://metadata.google.internal/reset", &response, &http_code));
+  ASSERT_EQ(http_code, 200);
+
+  size_t buflen = 200 * sizeof(char);
+  char* buffer = (char*)malloc(buflen);
+  ASSERT_STRNE(buffer, NULL);
+  BufferManager buf(buffer, buflen);
+  int errnop = 0;
+
+  struct group grp = {};
+  ASSERT_TRUE(GetGroupByName("demo", &grp, &buf, &errnop));
+  ASSERT_EQ(errnop, 0);
+}
+
+TEST(GetGroupByTest, GetGroupByGIDSucceeds) {
   string response;
   long http_code;
   ASSERT_TRUE(HttpGet("http://metadata.google.internal/reset", &response, &http_code));
@@ -449,28 +456,10 @@ TEST(FindGroupTest, FindGroupByGidSucceeds) {
   int errnop = 0;
 
   struct group grp = {};
-  grp.gr_gid = 123452;
-  ASSERT_TRUE(FindGroup(&grp, &buf, &errnop));
+  ASSERT_TRUE(GetGroupByGID(123452, &grp, &buf, &errnop));
   ASSERT_EQ(errnop, 0);
 }
 
-TEST(FindGroupTest, FindGroupByNameSucceeds) {
-  string response;
-  long http_code;
-  ASSERT_TRUE(HttpGet("http://metadata.google.internal/reset", &response, &http_code));
-
-  size_t buflen = 200 * sizeof(char);
-  char* buffer = (char*)malloc(buflen);
-  ASSERT_STRNE(buffer, NULL);
-  BufferManager buf(buffer, buflen);
-  int errnop;
-
-  const char* match = "demo";
-  struct group grp = {};
-  grp.gr_name = (char*)match;
-  ASSERT_TRUE(FindGroup(&grp, &buf, &errnop));
-}
-
 TEST(ParseJsonEmailTest, SuccessfullyParsesEmail) {
   string test_user =
       "{\"loginProfiles\":[{\"name\":\"foo@example.com\",\"posixAccounts\":["
